@@ -32,22 +32,27 @@
 #' @param .size Number of trials, zero or more.
 #' @param .prob Probability of success on each trial.
 #' @param .num_sims The number of randomly generated simulations you want.
+#' @param .return_tibble A logical value indicating whether to return the result
+#' as a tibble. Default is TRUE.
 #'
 #' @examples
 #' tidy_binomial()
 #' @return
 #' A tibble of randomly generated data.
-#'
+#' @name tidy_binomial
+NULL
 #' @export
-#'
+#' @rdname tidy_binomial
 
-tidy_binomial <- function(.n = 50, .size = 0, .prob = 1, .num_sims = 1) {
+tidy_binomial <- function(.n = 50, .size = 0, .prob = 1, .num_sims = 1,
+                          .return_tibble = TRUE) {
 
   # Tidyeval ----
   n <- as.integer(.n)
   num_sims <- as.integer(.num_sims)
   size <- as.numeric(.size)
   prob <- as.numeric(.prob)
+  ret_tbl <- as.logical(.return_tibble)
 
   # Checks ----
   if (!is.integer(n) | n < 0) {
@@ -84,17 +89,26 @@ tidy_binomial <- function(.n = 50, .size = 0, .prob = 1, .num_sims = 1) {
   qs <- seq(0, 1, (1 / (n - 1)))
   ps <- qs
 
-  df <- dplyr::tibble(sim_number = as.factor(x)) %>%
-    dplyr::group_by(sim_number) %>%
-    dplyr::mutate(x = list(1:n)) %>%
-    dplyr::mutate(y = list(stats::rbinom(n = n, size = size, prob = prob))) %>%
-    dplyr::mutate(d = list(density(unlist(y), n = n)[c("x", "y")] %>%
-      purrr::set_names("dx", "dy") %>%
-      dplyr::as_tibble())) %>%
-    dplyr::mutate(p = list(stats::pbinom(unlist(y), size = size, prob = prob))) %>%
-    dplyr::mutate(q = list(stats::qbinom(unlist(p), size = size, prob = prob))) %>%
-    tidyr::unnest(cols = c(x, y, d, p, q)) %>%
-    dplyr::ungroup()
+  # Create a data.table with one row per simulation
+  df <- data.table::CJ(sim_number = factor(1:num_sims), x = 1:n)
+
+  # Group the data by sim_number and add columns for x and y
+  df[, y := stats::rbinom(n = .N, size = size, prob = prob)]
+
+  # Compute the density of the y values and add columns for dx and dy
+  df[, c("dx", "dy") := density(y, n = n)[c("x", "y")], by = sim_number]
+
+  # Compute the p-values for the y values and add a column for p
+  df[, p := stats::pbinom(y, size = size, prob = prob)]
+
+  # Compute the q-values for the p-values and add a column for q
+  df[, q := stats::qbinom(p, size = size, prob = prob)]
+
+  if(.return_tibble){
+    df <- dplyr::as_tibble(df)
+  } else {
+    data.table::setkey(df, NULL)
+  }
 
   param_grid <- dplyr::tibble(.size, .prob)
 
@@ -104,6 +118,7 @@ tidy_binomial <- function(.n = 50, .size = 0, .prob = 1, .num_sims = 1) {
   attr(df, ".prob") <- .prob
   attr(df, ".n") <- .n
   attr(df, ".num_sims") <- .num_sims
+  attr(df, ".ret_tbl") <- .return_tibble
   attr(df, "tibble_type") <- "tidy_binomial"
   attr(df, "ps") <- ps
   attr(df, "qs") <- qs
