@@ -32,22 +32,29 @@
 #' @param .min A lower limit of the distribution.
 #' @param .max An upper limit of the distribution
 #' @param .num_sims The number of randomly generated simulations you want.
+#' @param .return_tibble A logical value indicating whether to return the result
+#' as a tibble. Default is TRUE.
 #'
 #' @examples
 #' tidy_uniform()
+#'
 #' @return
 #' A tibble of randomly generated data.
 #'
+#' @name tidy_uniform
+NULL
 #' @export
-#'
+#' @rdname tidy_uniform
 
-tidy_uniform <- function(.n = 50, .min = 0, .max = 1, .num_sims = 1) {
+tidy_uniform <- function(.n = 50, .min = 0, .max = 1, .num_sims = 1,
+                         .return_tibble = TRUE) {
 
   # Tidyeval ----
   n <- as.integer(.n)
   num_sims <- as.integer(.num_sims)
   max_val <- as.numeric(.max)
   min_val <- as.numeric(.min)
+  ret_tbl <- as.logical(.return_tibble)
 
   # Checks ----
   if (!is.integer(n) | n < 0) {
@@ -77,18 +84,28 @@ tidy_uniform <- function(.n = 50, .min = 0, .max = 1, .num_sims = 1) {
   qs <- seq(0, 1, (1 / (n - 1)))
   ps <- qs
 
-  df <- dplyr::tibble(sim_number = as.factor(x)) %>%
-    dplyr::group_by(sim_number) %>%
-    dplyr::mutate(x = list(1:n)) %>%
-    dplyr::mutate(y = list(stats::runif(n = n, min = min_val, max = max_val))) %>%
-    dplyr::mutate(d = list(density(unlist(y), n = n)[c("x", "y")] %>%
-      purrr::set_names("dx", "dy") %>%
-      dplyr::as_tibble())) %>%
-    dplyr::mutate(p = list(stats::punif(unlist(y), min = min_val, max = max_val))) %>%
-    dplyr::mutate(q = list(stats::qunif(unlist(p), min = min_val, max = max_val))) %>%
-    tidyr::unnest(cols = c(x, y, d, p, q)) %>%
-    dplyr::ungroup()
+  # Create a data.table with one row per simulation
+  df <- data.table::CJ(sim_number = factor(1:num_sims), x = 1:n)
 
+  # Group the data by sim_number and add columns for x and y
+  df[, y := stats::runif(n = .N, min = min_val, max = max_val)]
+
+  # Compute the density of the y values and add columns for dx and dy
+  df[, c("dx", "dy") := density(y, n = n)[c("x", "y")], by = sim_number]
+
+  # Compute the p-values for the y values and add a column for p
+  df[, p := stats::punif(y, min = min_val, max = max_val)]
+
+  # Compute the q-values for the p-values and add a column for q
+  df[, q := stats::qunif(p, min = min_val, max = max_val)]
+
+  if(.return_tibble){
+    df <- dplyr::as_tibble(df)
+  } else {
+    data.table::setkey(df, NULL)
+  }
+
+  # Create a tibble with the parameter grid
   param_grid <- dplyr::tibble(.min, .max)
 
   # Attach descriptive attributes to tibble
@@ -97,6 +114,7 @@ tidy_uniform <- function(.n = 50, .min = 0, .max = 1, .num_sims = 1) {
   attr(df, ".min") <- .min
   attr(df, ".n") <- .n
   attr(df, ".num_sims") <- .num_sims
+  attr(df, ".ret_tbl") <- .return_tibble
   attr(df, "tibble_type") <- "tidy_uniform"
   attr(df, "ps") <- ps
   attr(df, "qs") <- qs
